@@ -87,7 +87,7 @@ def load_omas_nc(filename, consistency_check=True, ):
     return ods
 
 
-def through_omas_nc(ods):
+def through_omas_nc(ods, method=['function', 'class_method'][1]):
     """
     Test save and load NetCDF
 
@@ -95,9 +95,11 @@ def through_omas_nc(ods):
 
     :return: ods
     """
-    if not os.path.exists(tempfile.gettempdir() + '/OMAS_TESTS/'):
-        os.makedirs(tempfile.gettempdir() + '/OMAS_TESTS/')
-    filename = tempfile.gettempdir() + '/OMAS_TESTS/test.nc'
-    save_omas_nc(ods, filename)
-    ods1 = load_omas_nc(filename)
+    filename = omas_testdir(__file__) + '/test.nc'
+    if method == 'function':
+        save_omas_json(ods, filename)
+        ods1 = load_omas_json(filename)
+    else:
+        ods.save(filename)
+        ods1 = ODS().load(filename)
     return ods1
