@@ -30,6 +30,31 @@ class constants(object):
     e = 1.6021766208e-19
 
 
+@add_to__ODS__
+def equilibrium_stored_energy(ods, update=True):
+    '''
+    Calculate MHD stored energy from equilibrium pressure and volume
+
+    :param ods: input ods
+
+    :param update: operate in place
+
+    :return: updated ods
+    '''
+    ods_n = ods
+    if not update:
+        from omas import ODS
+        ods_n = ODS().copy_attrs_from(ods)
+
+    for time_index in ods['equilibrium']['time_slice']:
+        pressure_equil = ods['equilibrium']['time_slice'][time_index]['profiles_1d']['pressure']
+        volume_equil = ods['equilibrium']['time_slice'][time_index]['profiles_1d']['volume']
+
+        ods_n['equilibrium.time_slice'][time_index]['.global_quantities.energy_mhd'] = 3.0 / 2.0 * numpy.trapz(pressure_equil, x=volume_equil)  # [J]
+
+    return ods_n
+
+
 @add_to__ODS__
 def core_profiles_consistent(ods, update=True, use_electrons_density=False):
     '''
@@ -560,6 +585,13 @@ def wall_add(ods, machine=None):
 
 @add_to__ODS__
 def equilibrium_consistent(ods):
+    '''
+    Calculate missing derived quantities for equilibrium IDS
+
+    :param ods: ODS to update in-place
+
+    :return: updated ods
+    '''
     for time_index in ods['equilibrium.time_slice']:
         eq = ods['equilibrium.time_slice'][time_index]
         eq1d = ods['equilibrium.time_slice'][time_index]['profiles_1d']
@@ -582,6 +614,10 @@ def equilibrium_consistent(ods):
         eq['profiles_2d.0.z'] = Z
         eq['profiles_2d.0.b_field_tor'] = fpol / R
 
+    equilibrium_stored_energy(ods, update=True)
+
+    return ods
+
 
 @add_to__ODS__
 def equilibrium_transpose_RZ(ods, flip_dims=False):
@@ -918,9 +954,82 @@ def cocos_transform(cocosin_index, cocosout_index):
     return transforms
 
 
+@add_to__ALL__
+def identify_cocos(B0, Ip, q, psi, clockwise_phi=None, a=None):
+    '''
+    Utility function to identify COCOS coordinate system
+    If multiple COCOS are possible, then all are returned.
+
+    :param B0: toroidal magnetic field (with sign)
+
+    :param Ip: plasma current (with sign)
+
+    :param q: safety factor profile (with sign) as function of psi
+
+    :param psi: poloidal flux as function of psi(with sign)
+
+    :param clockwise_phi: (optional) [True, False] if phi angle is defined clockwise or not
+                          This is required to identify odd Vs even COCOS
+                          Note that this cannot be determined from the output of a code.
+                          An easy way to determine this is to answer the question: is positive B0 clockwise?
+
+    :param a: (optional) flux surfaces minor radius as function of psi
+              This is required to identify 2*pi term in psi definition
+
+    :return: list with possible COCOS
+    '''
+
+    if clockwise_phi is None:
+        sigma_rpz = clockwise_phi
+    elif clockwise_phi:
+        sigma_rpz = +1
+    else:
+        sigma_rpz = -1
+
+    # return both even and odd COCOS if clockwise_phi is not provided
+    if sigma_rpz is None:
+        tmp = identify_cocos(B0, Ip, q, psi, True, a)
+        tmp.extend(identify_cocos(B0, Ip, q, psi, False, a))
+        return tmp
+
+    sigma_Ip = numpy.sign(Ip)
+    sigma_B0 = numpy.sign(B0)
+    sign_dpsi_pos = numpy.sign(gradient(psi))[0]
+    sign_q_pos = numpy.sign(q)[0]
+
+    sigma_Bp = sign_dpsi_pos / sigma_Ip
+    sigma_rhotp = sign_q_pos / (sigma_Ip * sigma_B0)
+
+    sigma2cocos = {(+1, +1, +1): 1,  # +Bp, +rpz, +rtp
+                   (+1, -1, +1): 2,  # +Bp, -rpz, +rtp
+                   (-1, +1, -1): 3,  # -Bp, +rpz, -rtp
+                   (-1, -1, -1): 4,  # -Bp, -rpz, -rtp
+                   (+1, +1, -1): 5,  # +Bp, +rpz, -rtp
+                   (+1, -1, -1): 6,  # +Bp, -rpz, -rtp
+                   (-1, +1, +1): 7,  # -Bp, +rpz, +rtp
+                   (-1, -1, +1): 8}  # -Bp, -rpz, +rtp
+
+    # identify 2*pi term in psi definition based on q estimate
+    if a is not None:
+        index = numpy.argmin(numpy.abs(q))
+        if index == 0:
+            index += 1
+        q_estimate = numpy.abs((pi * B0 * (a - a[0]) ** 2) / (psi - psi[0]))
+        if numpy.abs(q_estimate[index] - q[index]) < numpy.abs(q_estimate[index] / (2 * pi) - q[index]):
+            eBp = 1
+        else:
+            eBp = 0
+
+        return [sigma2cocos[(sigma_Bp, sigma_rpz, sigma_rhotp)] + 10 * eBp]
+
+    # return COCOS<10 as well as COCOS>10 if a is not provided
+    else:
+        return [sigma2cocos[(sigma_Bp, sigma_rpz, sigma_rhotp)], sigma2cocos[(sigma_Bp, sigma_rpz, sigma_rhotp)] + 10]
+
+
 @add_to__ALL__
 @contextmanager
-def omas_environment(ods, cocosio=None, coordsio=None, unitsio=None, **kw):
+def omas_environment(ods, cocosio=None, coordsio=None, unitsio=None, input_data_process_functions=None, xmlcodeparams=False, **kw):
     '''
     Provides environment for data input/output to/from OMAS
 
@@ -932,6 +1041,8 @@ def omas_environment(ods, cocosio=None, coordsio=None, unitsio=None, **kw):
 
     :param unitsio: True/False whether data read from OMAS should have units
 
+    :param xmlcodeparams: view code.parameters as an XML string while in this environment
+
     :param kw: extra keywords set attributes of the ods (eg. 'consistency_check', 'dynamic_path_creation', 'imas_version')
 
     :return: ODS with environment set
@@ -967,6 +1078,16 @@ def omas_environment(ods, cocosio=None, coordsio=None, unitsio=None, **kw):
     if unitsio is not None:
         ods.unitsio = unitsio
 
+    # set input_data_process_functions
+    if input_data_process_functions is not None:
+        from . import omas_core
+        bkp_input_data_process_functions = copy.copy(omas_core.input_data_process_functions)
+        omas_core.input_data_process_functions[:] = input_data_process_functions
+
+    # set code.parameters as XML string
+    if xmlcodeparams:
+        ods.codeparams2xml()
+
     try:
         if coordsio is not None:
             with omas_environment(coordsio, cocosio=cocosio):
@@ -975,12 +1096,24 @@ def omas_environment(ods, cocosio=None, coordsio=None, unitsio=None, **kw):
             yield ods
 
     finally:
+        # restore code.parameters as dictionary
+        if xmlcodeparams:
+            ods.codeparams2dict()
         # restore attributes
         ods.cocosio = bkp_cocosio
         ods.coordsio = bkp_coordsio
         ods.unitsio = bkp_unitsio
         for item in kw:
-            setattr(ods, item, bkp_args[item])
+            try:
+                setattr(ods, item, bkp_args[item])
+            except Exception as _excp:
+                # Add more user feedback, since use of consistency_check in an omas_environment can be confusing
+                if item == 'consistency_check':
+                    raise _excp.__class__(str(_excp) + '\nThe IMAS consistency was violated getting out of the omas_environment')
+                raise
+        # restore input_data_process_functions
+        if input_data_process_functions is not None:
+            omas_core.input_data_process_functions[:] = bkp_input_data_process_functions
 
 
 def generate_cocos_signals(structures=[], threshold=0, write=True, verbose=True):
@@ -1001,8 +1134,8 @@ def generate_cocos_signals(structures=[], threshold=0, write=True, verbose=True)
     """
     # units of entries currently in cocos_singals
     cocos_units = []
-    for item in cocos_signals:
-        if cocos_signals[item] == '?':
+    for item in _cocos_signals:
+        if _cocos_signals[item] == '?':
             continue
         info = omas_info_node(item)
         if len(info):  # info may have no length if nodes are deleted between IMAS versions
@@ -1023,13 +1156,24 @@ def generate_cocos_signals(structures=[], threshold=0, write=True, verbose=True)
     structures += cocos_structures
     structures = numpy.unique(structures)
 
-    from .omas_utils import _structures
+    from .omas_utils import _structures, _extra_structures
     from .omas_utils import i2o
     from .omas_core import ODS
-    ods = ODS()
-    out = {}
-    text = []
-    csig = ["""
+
+    # if generate_cocos_signals is run after omas has been used for something else
+    # (eg. when running test_examples) it may be that _extra_structures is not empty
+    # Thus, we clear _structures and _extra_structures to make sure that
+    # dict_structures() is not polluted by _extra_structures
+    _structures_bkp = copy.deepcopy(_structures)
+    _extra_structures_bkp = copy.deepcopy(_extra_structures)
+    try:
+        _structures.clear()
+        _extra_structures.clear()
+
+        ods = ODS()
+        out = {}
+        text = []
+        csig = ["""
 '''List of automatic COCOS transformations
 
 -------
@@ -1037,7 +1181,7 @@ def generate_cocos_signals(structures=[], threshold=0, write=True, verbose=True)
 # COCOS signals candidates are generated by running utilities/generate_cocos_signals.py
 # Running this script is useful to keep track of new signals that IMAS adds in new data structure releases
 #
-# In this file you are only allowed to edit/add entries to the `cocos_signals` dictionary
+# In this file you are only allowed to edit/add entries to the `_cocos_signals` dictionary
 # The comments indicate `#[ADD_OR_DELETE_SUGGESTION]# MATCHING_SCORE # RATIONALE_FOR_ADD_OR_DELETE`
 #
 # Proceed as follows:
@@ -1064,122 +1208,132 @@ def generate_cocos_signals(structures=[], threshold=0, write=True, verbose=True)
 #        transforms[None] = 1
 #
                 
-cocos_signals = {}
+_cocos_signals = {}
 """]
 
-    # loop over structures
-    for structure in structures:
-        text.extend(['', '# ' + structure.upper()])
-        csig.extend(['', '# ' + structure.upper()])
+        # loop over structures
+        for structure in structures:
+            text.extend(['', '# ' + structure.upper()])
+            csig.extend(['', '# ' + structure.upper()])
 
-        out[structure] = {}
-        ods[structure]
-        d = dict_structures(imas_version=omas_rcparams['default_imas_version'])
-        m = 0
-        # generate score and add reason for scoring
-        for item in sorted(list(_structures[(structure, omas_rcparams['default_imas_version'])].keys())):
-            item = i2o(item)
-            item_ = item
-            if any([item.endswith(k) for k in [':.values', ':.value', ':.data']]):
-                item_ = l2o(p2l(item)[:-2])
-            elif any([item.endswith(k) for k in ['.values', '.value', '.data']]):
-                item_ = l2o(p2l(item)[:-1])
-            m = max(m, len(item))
-            score = 0
-            rationale = []
-            if item.startswith(structure) and '_error_' not in item:
-                entry = "cocos_signals['%s']=" % i2o(item)
-                info = omas_info_node(item)
-                units = info.get('units', None)
-                data_type = info.get('data_type', None)
-                documentation = info.get('documentation', '')
-                if data_type in ['structure', 'STR_0D', 'struct_array']:
-                    continue
-                elif units in [None, 's']:
-                    out[structure].setdefault(-1, []).append((item, '[%s]' % units))
-                    continue
-                elif any([(item_.endswith('.' + k) or item_.endswith('_' + k) or '.' + k + '.' in item) for k in
-                          ['chi_squared', 'standard_deviation', 'weight', 'coefficients', 'r', 'z', 'beta_tor',
-                           'beta_pol', 'radial', 'rho_tor_norm', 'darea_drho_tor',
-                           'dvolume_drho_tor', 'ratio', 'fraction', 'rate', 'd', 'flux', 'v', 'b_field_max', 'width_tor']]):
-                    out[structure].setdefault(-1, []).append((item, p2l(item_)[-1]))
-                    continue
-                elif any([k in documentation for k in ['always positive']]):
-                    out[structure].setdefault(-1, []).append((item, documentation))
-                    continue
-                n = item.count('.')
-                for pnt, key in enumerate(p2l(item)):
-                    pnt = pnt / float(n)
-                    for case in ['q', 'ip', 'b0', 'phi', 'psi', 'f', 'f_df']:
-                        if key == case:
-                            rationale += [case]
-                            score += pnt
-                            break
-                    for case in ['q', 'j', 'phi', 'psi', 'ip', 'b', 'f', 'v', 'f_df']:
-                        if key.startswith('%s_' % case) and not any([key.startswith(k) for k in ['psi_norm']]):
-                            rationale += [case]
-                            score += pnt
-                            break
-                    for case in ['velocity', 'current', 'b_field', 'e_field', 'torque', 'momentum']:
-                        if case in key and key not in ['heating_current_drive']:
-                            rationale += [case]
-                            score += pnt
-                            break
-                    for case in ['_dpsi']:
-                        if case in key and case + '_norm' not in key:
-                            rationale += [case]
-                            score += pnt
-                            break
-                    for case in ['poloidal', 'toroidal', 'parallel', '_tor', '_pol', '_par', 'tor_', 'pol_', 'par_']:
-                        if ((key.endswith(case) or key.startswith(case)) and not any([key.startswith(k) for k in ['conductivity_', 'pressure_', 'rho_', 'length_']])):
-                            rationale += [case]
-                            score += pnt
-                            break
-                if units in cocos_units:
-                    if len(rationale):
-                        rationale += ['[%s]' % units]
-                        score += 1
-                out[structure].setdefault(score, []).append((item, '  '.join(rationale)))
+            out[structure] = {}
+            ods[structure]
+            d = dict_structures(imas_version=omas_rcparams['default_imas_version'])
+            m = 0
+            # generate score and add reason for scoring
+            for item in sorted(list(_structures[(structure, omas_rcparams['default_imas_version'])].keys())):
+                item = i2o(item)
+                item_ = item
+                if any([item.endswith(k) for k in [':.values', ':.value', ':.data']]):
+                    item_ = l2o(p2l(item)[:-2])
+                elif any([item.endswith(k) for k in ['.values', '.value', '.data']]):
+                    item_ = l2o(p2l(item)[:-1])
+                m = max(m, len(item))
+                score = 0
+                rationale = []
+                if item.startswith(structure) and '_error_' not in item:
+                    entry = "_cocos_signals['%s']=" % i2o(item)
+                    info = omas_info_node(item)
+                    units = info.get('units', None)
+                    data_type = info.get('data_type', None)
+                    documentation = info.get('documentation', '')
+                    if data_type in ['structure', 'STR_0D', 'struct_array']:
+                        continue
+                    elif units in [None, 's']:
+                        out[structure].setdefault(-1, []).append((item, '[%s]' % units))
+                        continue
+                    elif any([(item_.endswith('.' + k) or item_.endswith('_' + k) or '.' + k + '.' in item) for k in
+                              ['chi_squared', 'standard_deviation', 'weight', 'coefficients', 'r', 'z', 'beta_tor',
+                               'beta_pol', 'radial', 'rho_tor_norm', 'darea_drho_tor',
+                               'dvolume_drho_tor', 'ratio', 'fraction', 'rate', 'd', 'flux', 'v', 'b_field_max', 'width_tor']]):
+                        out[structure].setdefault(-1, []).append((item, p2l(item_)[-1]))
+                        continue
+                    elif any([k in documentation for k in ['always positive']]):
+                        out[structure].setdefault(-1, []).append((item, documentation))
+                        continue
+                    n = item.count('.')
+                    for pnt, key in enumerate(p2l(item)):
+                        pnt = pnt / float(n)
+                        for case in ['q', 'ip', 'b0', 'phi', 'psi', 'f', 'f_df']:
+                            if key == case:
+                                rationale += [case]
+                                score += pnt
+                                break
+                        for case in ['q', 'j', 'phi', 'psi', 'ip', 'b', 'f', 'v', 'f_df']:
+                            if key.startswith('%s_' % case) and not any([key.startswith(k) for k in ['psi_norm']]):
+                                rationale += [case]
+                                score += pnt
+                                break
+                        for case in ['velocity', 'current', 'b_field', 'e_field', 'torque', 'momentum']:
+                            if case in key and key not in ['heating_current_drive']:
+                                rationale += [case]
+                                score += pnt
+                                break
+                        for case in ['_dpsi']:
+                            if case in key and case + '_norm' not in key:
+                                rationale += [case]
+                                score += pnt
+                                break
+                        for case in ['poloidal', 'toroidal', 'parallel', '_tor', '_pol', '_par', 'tor_', 'pol_', 'par_']:
+                            if ((key.endswith(case) or key.startswith(case)) and not any([key.startswith(k) for k in ['conductivity_', 'pressure_', 'rho_', 'length_']])):
+                                rationale += [case]
+                                score += pnt
+                                break
+                    if units in cocos_units:
+                        if len(rationale):
+                            rationale += ['[%s]' % units]
+                            score += 1
+                    out[structure].setdefault(score, []).append((item, '  '.join(rationale)))
 
-        # generate output
-        for score in reversed(sorted(out[structure])):
-            for item, rationale in out[structure][score]:
+            # generate output
+            for score in reversed(sorted(out[structure])):
+                for item, rationale in out[structure][score]:
 
-                message = '       '
-                if cocos_signals.get(item, '?') == '?':
-                    if score > 0:
-                        message = '#[ADD?]'
-                    else:
+                    message = '       '
+                    if _cocos_signals.get(item, '?') == '?':
+                        if score > 0:
+                            message = '#[ADD?]'
+                        else:
+                            message = '#[DEL?]'
+                    elif score < 0:
                         message = '#[DEL?]'
-                elif score < 0:
-                    message = '#[DEL?]'
 
-                txt = ("cocos_signals['%s']=%s" % (item, repr(cocos_signals.get(item, '?')))).ljust(m + 20) + message + '# %f # %s' % (score, rationale)
-                text.append(txt)
-                if score > threshold or (item in cocos_signals and cocos_signals[item] != '?'):
-                    csig.append(txt)
+                    transform = _cocos_signals.get(item, '?')
+                    transform = None if transform is None else "'%s'" % transform
+                    txt = ("_cocos_signals['%s']=%s" % (item, transform)).ljust(m + 20) + message + '# %f # %s' % (score, rationale)
+                    text.append(txt)
+                    if score > threshold or (item in _cocos_signals and _cocos_signals[item] != '?'):
+                        csig.append(txt)
 
-        # print to screen (note that this prints ALL the entries, whereas omas_cocos.py only contains entries that score above a give threshold)
-        if verbose:
-            print('\n'.join(text) + '\n\n' + '-' * 20)
+            # print to screen (note that this prints ALL the entries, whereas omas_cocos.py only contains entries that score above a give threshold)
+            if verbose:
+                print('\n'.join(text) + '\n\n' + '-' * 20)
 
-    filename = os.path.abspath(str(os.path.dirname(__file__)) + '/omas_cocos.py')
-    if write:
-        # update omas_cocos.py
-        with open(filename, 'w') as f:
-            f.write(str('\n'.join(csig)))
-    else:
-        # check that omas_cocos.py file is up-to-date
-        with open(filename, 'r') as f:
-            tmp = str(f.read())
-        tmp1 = str('\n'.join(csig))
-        assert tmp == tmp1, 'COCOS signals are not up-to-date! Run `make cocos` to update the omas_cocos.py file.'
+        filename = os.path.abspath(str(os.path.dirname(__file__)) + '/omas_cocos.py')
+        if write:
+            # update omas_cocos.py
+            with open(filename, 'w') as f:
+                f.write(str('\n'.join(csig)))
+        else:
+            # check that omas_cocos.py file is up-to-date
+            with open(filename, 'r') as f:
+                original = str(f.read())
+            new = str('\n'.join(csig))
+            import difflib
+            diff = difflib.unified_diff(original, new)
+            assert original == new, 'COCOS signals are not up-to-date! Run `make cocos` to update the omas_cocos.py file.\n' + ''.join(diff)
 
-    return out
+        return out
+
+    finally:
+        _structures.clear()
+        _structures.update(_structures_bkp)
+        _extra_structures.clear()
+        _extra_structures.update(_extra_structures_bkp)
 
 
 # cocos_signals contains the IMAS locations and the corresponding `cocos_transform` function
-from .omas_cocos import cocos_signals as _cocos_signals
+from .omas_cocos import _cocos_signals
 
 
 # The CocosSignals class is just a dictionary that raises warnings when users access
