@@ -42,8 +42,18 @@ def dataset_description(ods):
     ods['dataset_description.data_entry.pulse'] = 100
     return ods
 
+
 @add_to_ODS
-def equilibrium(ods, time_index=0, include_profiles=True, include_phi=True, include_wall=True):
+def equilibrium(
+        ods,
+        time_index=0,
+        include_profiles=True,
+        include_phi=True,
+        include_psi=True,
+        include_wall=True,
+        include_q=True,
+        include_xpoint=False,
+):
     """
     Add sample equilibrium data
 
@@ -54,12 +64,27 @@ def equilibrium(ods, time_index=0, include_profiles=True, include_phi=True, incl
 
     :param include_profiles: bool
         Include 1D profiles of pressure, q, p', FF'
+        They are in the sample set, so not including them means deleting them.
 
     :param include_phi: bool
         Include 1D and 2D profiles of phi (toroidal flux, for calculating rho)
+        This is in the sample set, so not including it means deleting it.
+
+    :param include_psi: bool
+        Include 1D and 2D profiles of psi (poloidal flux)
+        This is in the sample set, so not including it means deleting it.
 
     :param include_wall: bool
         Include the first wall
+        This is in the sample set, so not including it means deleting it.
+
+    :param include_q: bool
+        Include safety factor
+        This is in the sample set, so not including it means deleting it.
+
+    :param include_xpoint: bool
+        Include X-point R-Z coordinates
+        This is not in the sample set, so including it means making it up
 
     :return: ODS instance with equilibrium data added
         Since the original is modified, it is not necessary to catch the return, but it may be convenient to do so in
@@ -69,6 +94,9 @@ def equilibrium(ods, time_index=0, include_profiles=True, include_phi=True, incl
     eq = load_omas_json(imas_json_dir + '/../samples/sample_equilibrium_ods.json', consistency_check=False)
 
     phi = eq['equilibrium.time_slice.0.profiles_1d.phi']
+    psi = eq['equilibrium.time_slice.0.profiles_1d.psi']
+    q = eq['equilibrium.time_slice.0.profiles_1d.q']
+
     if not include_profiles:
         del eq['equilibrium.time_slice.0.profiles_1d']
 
@@ -79,13 +107,33 @@ def equilibrium(ods, time_index=0, include_profiles=True, include_phi=True, incl
     else:
         eq['equilibrium.time_slice.0.profiles_1d.phi'] = phi
 
+    if not include_psi:
+        if 'profiles_1d' in eq['equilibrium.time_slice.0'] and 'psi' in eq['equilibrium.time_slice.0.profiles_1d']:
+            del eq['equilibrium.time_slice.0.profiles_1d.psi']
+        del eq['equilibrium.time_slice.0.profiles_2d.0.psi']
+    else:
+        eq['equilibrium.time_slice.0.profiles_1d.psi'] = psi
+
+    if not include_q:
+        if 'profiles_1d' in eq['equilibrium.time_slice.0'] and 'q' in eq['equilibrium.time_slice.0.profiles_1d']:
+            del eq['equilibrium.time_slice.0.profiles_1d.q']
+    else:
+        eq['equilibrium.time_slice.0.profiles_1d.q'] = q
+
     if not include_wall:
         del eq['wall']
 
-    ods['equilibrium.time_slice'][time_index].update(eq['equilibrium.time_slice.0'])
+    if include_xpoint:
+        eq['equilibrium.time_slice'][0]['boundary.x_point.0.r'] = 1.304
+        eq['equilibrium.time_slice'][0]['boundary.x_point.0.z'] = -1.222
+
     ods['equilibrium.time_slice'][time_index]['time'] = float(time_index)
+    ods['equilibrium.time_slice'][time_index].update(eq['equilibrium.time_slice.0'])
     ods['equilibrium.vacuum_toroidal_field.r0'] = eq['equilibrium.vacuum_toroidal_field.r0']
-    ods.set_time_array('equilibrium.vacuum_toroidal_field.b0', time_index, eq['equilibrium.vacuum_toroidal_field.b0'][0])
+    ods.set_time_array(
+        'equilibrium.vacuum_toroidal_field.b0', time_index, eq['equilibrium.vacuum_toroidal_field.b0'][0]
+    )
+    ods['equilibrium.time'] = ods['equilibrium.time_slice[:].time']
 
     return ods
 
@@ -239,10 +287,10 @@ def magnetics(ods):
     zf = z0 + afl * numpy.sin(angle_fl)
 
     for i in range(nbp):
-        ods['magnetics.bpol_probe'][i]['identifier'] = 'FAKE bpol probe {}'.format(i)
-        ods['magnetics.bpol_probe'][i]['position.r'] = rp[i]
-        ods['magnetics.bpol_probe'][i]['position.z'] = zp[i]
-        ods['magnetics.bpol_probe'][i]['position.phi'] = 6.5
+        ods['magnetics.b_field_pol_probe'][i]['identifier'] = 'FAKE bpol probe {}'.format(i)
+        ods['magnetics.b_field_pol_probe'][i]['position.r'] = rp[i]
+        ods['magnetics.b_field_pol_probe'][i]['position.z'] = zp[i]
+        ods['magnetics.b_field_pol_probe'][i]['position.phi'] = 6.5
 
     for i in range(nfl):
         ods['magnetics.flux_loop'][i]['identifier'] = 'FAKE flux loop {}'.format(i)
@@ -402,6 +450,9 @@ def gas_injection(ods):
     ods['gas_injection.pipe.1.exit_position.z'] = 1.1
     ods['gas_injection.pipe.1.exit_position.phi'] = 6.5
     ods['gas_injection.pipe.1.valve.0.identifier'] = 'FAKE_GAS_VALVE_B'
+    ods['gas_injection.pipe.1.second_point.r'] = 1.63
+    ods['gas_injection.pipe.1.second_point.z'] = 1.08
+    ods['gas_injection.pipe.1.second_point.phi'] = 6.5
 
     ods['gas_injection.pipe.2.name'] = 'FAKE_GAS_C'
     ods['gas_injection.pipe.2.exit_position.r'] = 2.1
@@ -410,3 +461,158 @@ def gas_injection(ods):
     # This one deliberately doesn't have a phi angle defined, for testing purposes.
 
     return ods
+
+
+@add_to_ODS
+def langmuir_probes(ods):
+    """
+    Adds some FAKE Langmuir probe locations so the overlay plot will work in tests.
+    :param ods: ODS instance
+
+    :return: ODS instance
+        The data are also written to the original, so you don't have to catch the return
+    """
+
+    ods['langmuir_probes.time'] = numpy.array([0])
+
+    ods['langmuir_probes.embedded.0.identifier'] = 0
+    ods['langmuir_probes.embedded.0.name'] = 'p1'
+    ods['langmuir_probes.embedded.0.position.r'] = 0.9
+    ods['langmuir_probes.embedded.0.position.z'] = 0.1
+    ods['langmuir_probes.embedded.0.position.phi'] = 0
+
+    ods['langmuir_probes.embedded.1.identifier'] = -1
+    ods['langmuir_probes.embedded.1.name'] = 'p23'
+    ods['langmuir_probes.embedded.1.position.r'] = 0.9
+    ods['langmuir_probes.embedded.1.position.z'] = -0.9
+    ods['langmuir_probes.embedded.1.position.phi'] = numpy.NaN
+
+    ods['langmuir_probes.embedded.2.identifier'] = -2
+    ods['langmuir_probes.embedded.2.name'] = 'blah'
+    ods['langmuir_probes.embedded.2.position.r'] = 1.5
+    ods['langmuir_probes.embedded.2.position.z'] = -1.25
+    ods['langmuir_probes.embedded.2.position.phi'] = numpy.NaN
+
+    ods['langmuir_probes.embedded.3.identifier'] = -3
+    ods['langmuir_probes.embedded.3.name'] = 'donkey!'
+    ods['langmuir_probes.embedded.3.position.r'] = 1.525
+    ods['langmuir_probes.embedded.3.position.z'] = -1.25
+    ods['langmuir_probes.embedded.3.position.phi'] = numpy.NaN
+
+    ods['langmuir_probes.embedded.4.identifier'] = -4
+    ods['langmuir_probes.embedded.4.name'] = 'zzz'
+    ods['langmuir_probes.embedded.4.position.r'] = 1.4
+    ods['langmuir_probes.embedded.4.position.z'] = 1.4
+    ods['langmuir_probes.embedded.4.position.phi'] = numpy.pi
+
+    ods['langmuir_probes.embedded.5.identifier'] = -5
+    ods['langmuir_probes.embedded.5.name'] = "it's just a test"
+    ods['langmuir_probes.embedded.5.position.r'] = 2.45
+    ods['langmuir_probes.embedded.5.position.z'] = 0.25
+    ods['langmuir_probes.embedded.5.position.phi'] = numpy.NaN
+
+    return ods
+
+
+@add_to_ODS
+def wall(ods):
+    """
+    Adds some FAKE wall data to support testing. The wall is approximately DIII-D shaped, but I didn't try very hard.
+
+    :param ods: ODS instance
+
+    :return: ODS instance
+        Edits are done in-place, so you don't have to catch the return (but you can if you want to!)
+    """
+    ods['wall.description_2d[0].limiter.type.description'] = 'first wall'
+    ods['wall.description_2d[0].limiter.type.index'] = 0
+    ods['wall.description_2d[0].limiter.type.name'] = 'first_wall'
+    ods['wall.description_2d[0].limiter.unit[0].outline.r'] = [
+        1.0, 1.0, 1.3, 1.4, 1.6, 2.15, 2.35, 2.35, 2.15, 1.800, 1.350, 1.35, 1.10, 1.00, 1.0
+    ]
+    ods['wall.description_2d[0].limiter.unit[0].outline.z'] = [
+        0.0, 1.4, 1.4, 1.3, 1.1, 1.00, 0.50, -0.5, -1.0, -1.25, -1.25, -1.4, -1.4, -1.3, 0.0
+    ]
+
+    return ods
+
+
+@add_to_ODS
+def pulse_schedule(ods_):
+    """
+    Adds some FAKE control target data to support testing.
+
+    :param ods: ODS instance
+
+    :return: ODS instance
+        Edits are done in-place, so you don't have to catch the return (but you can if you want to!)
+    """
+    import numpy as np
+
+    def add_position_control(ods):
+        """Adds sample data for the position control subset"""
+        bdry = ods['pulse_schedule.position_control.boundary_outline']
+
+        # These data are sampled from DIII-D#161558 at the following times:
+        t = np.array([0.1, 0.52, 0.99, 1.29, 1.46, 2.01, 3.91, 5.97, 6.6, 6.9])  # s
+        bdry[0]['r.reference.data'] = np.array([2.31, 2.27, 2.27, 2.27, 2.27, 2.27, 2.27, 2.27, 2.25, 2.25])  # m
+        bdry[0]['z.reference.data'] = np.array([0., 0., 0., 0., 0., 0., 0., 0., 0., 0.])
+        bdry[1]['r.reference.data'] = np.array([2.21, 2.17, 2.17, 2.16, 2.16, 2.15, 2.16, 2.15, 2.12, 2.12])
+        bdry[1]['z.reference.data'] = np.array([0.43, 0.43, 0.43, 0.43, 0.43, 0.43, 0.43, 0.43, 0.43, 0.43])
+        bdry[2]['r.reference.data'] = np.array([1.9, 1.9, 1.9, 1.88, 1.87, 1.87, 1.87, 1.87, 1.83, 1.83])
+        bdry[2]['z.reference.data'] = np.array([0.81, 0.81, 0.81, 0.78, 0.76, 0.76, 0.76, 0.76, 0.69, 0.69])
+        bdry[3]['r.reference.data'] = np.array([1.52, 1.52, 1.52, 1.52, 1.52, 1.52, 1.52, 1.52, 1.52, 1.52])
+        bdry[3]['z.reference.data'] = np.array([0.84, 0.91, 0.91, 0.87, 0.84, 0.83, 0.83, 0.83, 1.35, 1.35])
+        bdry[4]['r.reference.data'] = np.array([1.44, 1.39, 1.39, 1.41, 1.43, 1.43, 1.43, 1.43, 1.1, 1.1])
+        bdry[4]['z.reference.data'] = np.array([0.8, 0.86, 0.86, 0.83, 0.81, 0.81, 0.81, 0.81, 1.21, 1.21])
+        bdry[5]['r.reference.data'] = np.array([1.32, 1.28, 1.28, 1.3, 1.31, 1.32, 1.32, 1.32, 1.07, 1.07])
+        bdry[5]['z.reference.data'] = np.array([0.73, 0.77, 0.77, 0.75, 0.74, 0.73, 0.73, 0.73, 0.99, 0.99])
+        bdry[6]['r.reference.data'] = np.array([0.92, 1.16, 1.16, 1.16, 1.17, 1.17, 1.17, 1.17, 0.92, 0.92])
+        bdry[6]['z.reference.data'] = np.array([0., 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0., 0.])
+        bdry[7]['r.reference.data'] = np.array([1.15, 1.1, 1.1, 1.1, 1.1, 1.1, 1.1, 1.1, 1.2, 1.2])
+        bdry[7]['z.reference.data'] = np.array([0.5, 0.16, 0.16, 0.16, 0.16, 0.16, 0.16, 0.16, 0.5, 0.5])
+        bdry[8]['r.reference.data'] = np.array([1.08, 1.1, 1.1, 1.1, 1.1, 1.1, 1.1, 1.1, 1.08, 1.08])
+        bdry[8]['z.reference.data'] = np.array([-0.5, -0.16, -0.16, -0.16, -0.16, -0.16, -0.16, -0.16, -0.5, -0.5])
+        bdry[9]['r.reference.data'] = np.array([1.23, 1.12, 1.12, 1.14, 1.14, 1.14, 1.14, 1.14, 1.23, 1.23])
+        bdry[9]['z.reference.data'] = np.array([-0.78, -0.5, -0.5, -0.5, -0.5, -0.5, -0.5, -0.5, -0.79, -0.79])
+        bdry[10]['r.reference.data'] = np.array([1.94, 1.18, 1.18, 1.21, 1.22, 1.22, 1.22, 1.22, 1.92, 1.92])
+        bdry[10]['z.reference.data'] = np.array([-0.88, -0.81, -0.81, -0.79, -0.79, -0.79, -0.79, -0.79, -0.85, -0.85])
+        bdry[11]['r.reference.data'] = np.array([2.23, 1.89, 1.89, 1.89, 1.89, 1.89, 1.89, 1.89, 2.18, 2.18])
+        bdry[11]['z.reference.data'] = np.array([-0.43, -0.8, -0.8, -0.8, -0.8, -0.8, -0.8, -0.8, -0.43, -0.43])
+
+        for i in range(12):
+            bdry[i]['r.reference_type'] = bdry[i]['z.reference_type'] = 1
+            bdry[i]['r.reference_name'] = bdry[i]['z.reference_name'] = 'seg{}'.format(i)
+            bdry[i]['r.reference.time'] = bdry[i]['z.reference.time'] = t
+
+        strk = ods['pulse_schedule.position_control.strike_point']
+        strk[0]['r.reference.data'] = np.array([np.NaN, 1.35, 1.35, 1.35, 1.35, 1.35, 1.35, 1.35, np.NaN, np.NaN])
+        strk[0]['z.reference.data'] = np.array(
+            [np.NaN, -1.35, -1.35, -1.35, -1.35, -1.35, -1.35, -1.35, np.NaN, np.NaN]
+        )
+        strk[1]['r.reference.data'] = np.array([np.NaN, 1.02, 1.02, 1.02, 1.02, 1.02, 1.02, 1.02, np.NaN, np.NaN])
+        strk[1]['z.reference.data'] = np.array([np.NaN, -1.2, -1.2, -1.2, -1.2, -1.2, -1.2, -1.2, np.NaN, np.NaN])
+
+        for i in range(2):
+            strk[i]['r.reference_type'] = strk[i]['z.reference_type'] = 1
+            strk[i]['r.reference_name'] = strk[i]['z.reference_name'] = 'strk{}'.format(i)
+            strk[i]['r.reference.time'] = strk[i]['z.reference.time'] = t
+
+        xpt = ods['pulse_schedule.position_control.x_point']
+        xpt[0]['r.reference.data'] = np.array([1.1, 1.34, 1.34, 1.43, 1.43, 1.43, 1.43, 1.43, 1.13, 1.13])
+        xpt[0]['z.reference.data'] = np.array([-1.6, -1.21, -1.21, -1.15, -1.15, -1.15, -1.15, -1.15, -1.41, -1.41])
+        xpt[1]['r.reference.data'] = np.array([np.NaN] * len(t))
+        xpt[1]['z.reference.data'] = np.array([np.NaN] * len(t))
+
+        for i in range(2):
+            xpt[i]['r.reference_type'] = xpt[i]['z.reference_type'] = 1
+            xpt[i]['r.reference_name'] = xpt[i]['z.reference_name'] = 'strk{}'.format(i)
+            xpt[i]['r.reference.time'] = xpt[i]['z.reference.time'] = t
+
+        ods['pulse_schedule.time'] = t
+
+        return
+
+    add_position_control(ods_)
+
+    return ods_
