@@ -40,6 +40,7 @@ import uncertainties
 import uncertainties.unumpy as unumpy
 from uncertainties.unumpy import nominal_values, std_devs, uarray
 from uncertainties import ufloat
+import ast
 
 # xarrays: avoid loading xarrays upfront since it can be slow and it is not always used
 import xarray
@@ -99,18 +100,22 @@ class IMAS_versions(OrderedDict):
     '''
     dictionary with list of IMAS version and their sub-folder name in the imas_json_dir
     '''
-    pass
+
+    def __init__(self, mode='all'):
+        OrderedDict.__init__(self)
+        if mode in ['all', 'named']:
+            # first `develop/3` and other branches
+            for _item in list(map(lambda x: os.path.basename(x), sorted(glob.glob(imas_json_dir + os.sep + '*')))):
+                if not _item.startswith('3'):
+                    self[_item.replace('_', '.')] = _item
+        if mode in ['all', 'tagged']:
+            # next all tagged versions sorted by version number
+            for _item in list(map(lambda x: os.path.basename(x), sorted(glob.glob(imas_json_dir + os.sep + '*')))):
+                if _item.startswith('3'):
+                    self[_item.replace('_', '.')] = _item
 
 
 imas_versions = IMAS_versions()
-# first `develop/3` and other branches
-for _item in list(map(lambda x: os.path.basename(x), sorted(glob.glob(imas_json_dir + os.sep + '*')))):
-    if not _item.startswith('3'):
-        imas_versions[_item.replace('_', '.')] = _item
-# next all tagged versions sorted by version number
-for _item in list(map(lambda x: os.path.basename(x), sorted(glob.glob(imas_json_dir + os.sep + '*')))):
-    if _item.startswith('3'):
-        imas_versions[_item.replace('_', '.')] = _item
 
 if 'OMAS_IMAS_VERSION' in os.environ:
     _default_imas_version = os.environ['OMAS_IMAS_VERSION']
@@ -147,7 +152,9 @@ omas_rcparams.update({
                                                                        'tmp',
                                                                        'OMAS_FAKE_IMAS_DIR'])),
     'allow_fake_imas_fallback': bool(int(os.environ.get('OMAS_ALLOW_FAKE_IMAS_FALLBACK', '0'))),
-    'default_imas_version': _default_imas_version
+    'default_imas_version': _default_imas_version,
+    'default_mongo_server': 'mongodb+srv://{user}:{pass}@omasdb-xymmt.mongodb.net',
+    'pickle_protocol': 2  # pickle.HIGHEST_PROTOCOL (`2` is used to ensure Python 3-->2 compatibility)
 })
 
 
@@ -193,3 +200,19 @@ add_datastructures['info'] = {
         "description": "run number"
     }
 }
+
+
+def omas_testdir(filename_topic=''):
+    '''
+    Return path to temporary folder where OMAS TEST file are saved/loaded
+
+    NOTE: If directory does not exists it is created
+
+    :return: string with path to OMAS TEST folder
+    '''
+    if filename_topic:
+        filename_topic = os.path.splitext(os.path.split(filename_topic)[-1])[0] + '/'
+    tmp = tempfile.gettempdir() + '/OMAS_TESTS/' + filename_topic
+    if not os.path.exists(tmp):
+        os.makedirs(tmp)
+    return tmp
