@@ -11,9 +11,9 @@ Test script for omas/omas_core.py
 -------
 """
 
-# Basic imports
 from __future__ import print_function, division, unicode_literals
 import unittest
+import os
 import numpy
 from pprint import pprint
 
@@ -24,27 +24,9 @@ from omas.omas_setup import *
 
 class TestOmasCore(unittest.TestCase):
     """
-    Test suite for omas_physics.py
+    Test suite for omas_core.py
     """
 
-    # Flags to edit while testing
-    verbose = False  # Spammy, but occasionally useful for debugging a weird problem
-
-    # Utilities for this test
-    def printv(self, *arg):
-        """Utility for tests to use"""
-        if self.verbose:
-            print(*arg)
-
-    def setUp(self):
-        test_id = self.id()
-        test_name = '.'.join(test_id.split('.')[-2:])
-        self.printv('{}...'.format(test_name))
-
-    def tearDown(self):
-        test_name = '.'.join(self.id().split('.')[-2:])
-        self.printv('    {} done.'.format(test_name))
-
     def test_misc(self):
         ods = ODS()
         # check effect of disabling dynamic path creation
@@ -136,6 +118,12 @@ class TestOmasCore(unittest.TestCase):
         # check writing setting an xarray.DataArray
         ods2['equilibrium.time_slice[2].profiles_1d.q'] = xarray.DataArray(uarray([0., 1., 2., 3.], [0, .1, .2, .3]), coords={'x': [1, 2, 3, 4]}, dims=['x'])
 
+    def test_dynamic_set_nonzero_array_index(self):
+        ods = ODS()
+        ods.consistency_check = False
+        ods.dynamic_path_creation = True
+        self.assertRaises(IndexError, ods.__setitem__, 'something[10]', 5)
+
     def test_coordinates(self):
         ods = ods_sample()
         assert (len(ods.list_coordinates()) > 0)
@@ -214,6 +202,26 @@ class TestOmasCore(unittest.TestCase):
         ods['dataset_description'].satisfy_imas_requirements()
         assert ods['dataset_description.ids_properties.homogeneous_time'] is not None
 
+    def test_dynamic_set_existing_list_nonzero_array_index(self):
+        ods = ODS()
+        ods.consistency_check = False
+        ods.dynamic_path_creation = 'dynamic_array_structures'
+        ods['something[0]'] = 5
+        ods['something[7]'] = 10
+        assert ods['something[0]'] == 5
+        assert ods['something[7]'] == 10
+
+    def test_set_nonexisting_array_index(self):
+        ods = ODS()
+        ods.consistency_check = False
+        ods.dynamic_path_creation = False
+        self.assertRaises(IndexError, ods.__setitem__, 'something.[10]', 5)
+
+    def test_force_type(self):
+        ods = ODS()
+        ods['core_profiles.profiles_1d'][0]['ion'][0]['z_ion'] = 1
+        assert isinstance(ods['core_profiles.profiles_1d'][0]['ion'][0]['z_ion'], float)
+
     def test_address_structures(self):
         ods = ODS()
 
@@ -242,11 +250,11 @@ class TestOmasCore(unittest.TestCase):
 
     def test_version(self):
         ods = ODS(imas_version='3.20.0')
-        ods['ec_antennas.antenna.0.power'] = 1.0
+        ods['ec_antennas.antenna.0.power.data'] = [1.0]
 
         try:
             ods = ODS(imas_version='3.21.0')
-            ods['ec_antennas.antenna.0.power'] = 1.0
+            ods['ec_antennas.antenna.0.power.data'] = [1.0]
             raise AssertionError('3.21.0 should not have `ec_antennas.antenna.0.power`')
         except LookupError:
             pass
@@ -306,6 +314,89 @@ class TestOmasCore(unittest.TestCase):
         ods2['equilibrium.vacuum_toroidal_field.r0'] += 1
         assert ods['equilibrium.vacuum_toroidal_field.r0'] + 1 == ods2['equilibrium.vacuum_toroidal_field.r0']
 
+    def test_saveload(self):
+        ods = ODS()
+        ods.sample_equilibrium()
+        ods.save('test.pkl')
+
+    def test_input_data_process_functions(self):
+        def robust_eval(string):
+            import ast
+            try:
+                return ast.literal_eval(string)
+            except:
+                return string
+
+        ods = ODS(consistency_check=False)
+        with omas_environment(ods, input_data_process_functions=[robust_eval]):
+            ods['int'] = '1'
+            ods['float'] = '1.0'
+            ods['basestring'] = 'bla'
+            ods['complex'] = '2+1j'
+        for item in ods:
+            assert isinstance(ods[item], eval(item))
+
+    def test_conversion_after_assignement(self):
+        ods = ODS(consistency_check=False)
+        ods['bla'] = 5
+        try:
+            ods[0] = 4
+            raise AssertionError('Convertion of dict to list should not be allowed')
+        except TypeError:
+            pass
+
+        del ods['bla']
+        ods[0] = 4
+        try:
+            ods['bla'] = 5
+            raise AssertionError('Convertion of list to dict should not be allowed')
+        except TypeError:
+            pass
+
+    def test_codeparameters(self):
+        ods = ODS()
+        ods['equilibrium.code.parameters'] = CodeParameters(imas_json_dir + '/../samples/input_gray.xml')
+
+        tmp = {}
+        tmp.update(ods['equilibrium.code.parameters'])
+        ods['equilibrium.code.parameters'] = tmp
+        assert isinstance(ods['equilibrium.code.parameters'], CodeParameters)
+
+        with omas_environment(ods, xmlcodeparams=True):
+            assert isinstance(ods['equilibrium.code.parameters'], basestring)
+        assert isinstance(ods['equilibrium.code.parameters'], CodeParameters)
+
+        # test that dynamic creation of .code.parameters makes it a CodeParameters object
+        ods = ODS()
+        ods['equilibrium.code.parameters']['param1'] = 1
+        assert isinstance(ods['equilibrium.code.parameters'], CodeParameters)
+
+        # test saving of code_parameters in json format
+        ods.save(tempfile.gettempdir() + '/ods_w_codeparams.json')
+
+        # test that loading of data with code.parameters results in a CodeParameters object
+        ods = ODS()
+        ods.load(tempfile.gettempdir() + '/ods_w_codeparams.json')
+        assert isinstance(ods['equilibrium.code.parameters'], CodeParameters)
+
+        # test loading CodeParameters from a json
+        ods = ODS().load(imas_json_dir + '/../samples/ods_w_code_parameters.json')
+        # test traversing ODS and code parameters with OMAS syntax
+        assert ods['ec_launchers.code.parameters.launcher.0.mharm'] == 2
+        # test that sub-tree elements of code parameters are also of CodeParameters class
+        assert isinstance(ods['ec_launchers.code.parameters.launcher'], CodeParameters)
+        # test to_string and from_string methods
+        with omas_environment(ods, xmlcodeparams=True):
+            code_parameters_string = ods['ec_launchers.code.parameters']
+            tmp = CodeParameters().from_string(code_parameters_string)
+            assert isinstance(tmp['launcher'], CodeParameters)
+            assert tmp['launcher.0.mharm'] == 2
+        # test that CodeParameters are restored after xmlcodeparams=True environment
+        assert isinstance(ods['ec_launchers.code.parameters'], CodeParameters)
+        assert isinstance(ods['ec_launchers.code.parameters.launcher'], CodeParameters)
+        assert isinstance(ods['ec_launchers.code.parameters.launcher.0'], CodeParameters)
+        assert len(ods['ec_launchers.code.parameters.launcher.0']) == 2
+
 
 if __name__ == '__main__':
     suite = unittest.TestLoader().loadTestsFromTestCase(TestOmasCore)
