@@ -13,15 +13,17 @@ Test script for omas/omas_plot.py
 
 # Basic imports
 from __future__ import print_function, division, unicode_literals
-import os
 import unittest
+import os
 import numpy
 import warnings
 import copy
 
-# Plot imports
-import matplotlib as mpl
-from matplotlib import pyplot as plt
+# Use Agg backend to avoid opening up figures
+import matplotlib
+
+matplotlib.use('Agg')
+from matplotlib import pyplot
 
 # OMAS imports
 from omas import *
@@ -33,37 +35,17 @@ class TestOmasPlot(unittest.TestCase):
     Test suite for omas_plot.py
     """
 
-    # Flags to edit while testing
-    show_plots = False  # This will get in the way of automatic testing
-    keep_plots_open = False
-    verbose = False  # Spammy, but occasionally useful for debugging a weird problem
-
     # Sample data for use in tests
     ods = ods_sample()
 
-    # Utilities for this test
-    def printv(self, *arg):
-        """Utility for tests to use"""
-        if self.verbose:
-            print(*arg)
-
     def setUp(self):
         test_id = self.id()
         test_name = '.'.join(test_id.split('.')[-2:])
         if test_name not in ['TestOmasPlot.test_ch_count']:
-            self.fig = plt.figure(test_name)
-        self.printv('{}...'.format(test_name))
+            self.fig = pyplot.figure(test_name)
 
     def tearDown(self):
-        test_name = '.'.join(self.id().split('.')[-2:])
-        self.printv('    {} done.'.format(test_name))
-        if not self.keep_plots_open:
-            plt.close()
-
-    @classmethod
-    def tearDownClass(cls):
-        if cls.show_plots:
-            plt.show()
+        pyplot.close()
 
     def test_quantity(self):
         self.ods.plot_quantity('core_profiles.profiles_1d.0.electrons.density_thermal', '$n_e$', lw=2)
@@ -112,7 +94,7 @@ class TestOmasPlot(unittest.TestCase):
         e = 0.1 + y * 0.01 + x * 0.01
         u = unumpy.uarray(y, e)
 
-        ax = plt.gca()
+        ax = pyplot.gca()
         ub1 = uband(x, u, ax)
         ub2 = uband(x, -u, fill_kw=dict(alpha=0.15, color='k'), color='r')
         assert ub1 != ub2
@@ -133,7 +115,6 @@ class TestOmasPlot(unittest.TestCase):
 
     def test_gas_arrow(self):
         from omas.omas_plot import gas_arrow
-        self.printv('  gas_arrow ods.cocos = {}'.format(self.ods.cocos))
         # Explicitly test the direction keyword
         gas_arrow(self.ods, 1.5, 0.0, direction=0, color='k')
         gas_arrow(self.ods, 1.5, 0.0, direction=numpy.pi / 2, color='gray')
@@ -154,32 +135,98 @@ class TestOmasPlot(unittest.TestCase):
         assert geo_type_lookup(1, 'pf_active', imas_version='99.99.99', reverse=False) is None
 
     # Equilibrium plots
-    def test_eqcx(self):
+    def test_eqcx_basic(self):
+        """Our basic ods comes with eq data, so try to just plot that thing"""
         self.ods.plot_equilibrium_CX()
-        ods2 = ODS().sample_equilibrium(include_profiles=False, include_phi=True, include_wall=False)
-        ods2.plot_equilibrium_CX()  # Should be vs. rho this time
-        ods2.sample_equilibrium(time_index=1, include_profiles=False, include_phi=False, include_wall=False).plot_equilibrium_CX()  # Get wall from slice 0
-        plt.figure('TestOmasPlot.test_eqcx missing wall')
-        ODS().sample_equilibrium(include_profiles=True, include_phi=False, include_wall=False).plot_equilibrium_CX()  # No wall
+
+    def test_eqcx_data_availability_variations(self):
+        """Plot all the equilibrium contour quantity options with all the combinations of available data"""
+        cq_options = ['rho', 'psi', 'phi', 'q']
+        for iwall in [True, False]:
+            for ipsi in [True, False]:
+                for iphi in [True, False]:
+                    for iprof in [True, False]:
+                        for iq in [True, False]:
+                            ods = ODS().sample_equilibrium(
+                                include_profiles=iprof,
+                                include_phi=iphi,
+                                include_psi=ipsi,
+                                include_wall=iwall,
+                                include_q=iq,
+                                include_xpoint=ipsi,  # This doesn't need an independent scan
+                            )
+                            if iprof and not ipsi:
+                                # Try to trip up the X-point plotter by putting in an incomplete definition. This
+                                # access attempt should cause the number of X-points to be interpreted as 1, even though
+                                # there is no data and so the X-point is not defined.
+                                ods['equilibrium.time_slice.0.boundary.x_point.0']
+
+                            for cqo in cq_options:
+                                ods.plot_equilibrium_CX(contour_quantity=cqo, allow_fallback=True)
+
+        # Test for disallowed fallback
+        ods = ODS().sample_equilibrium(include_psi=False, include_phi=True)
+        with self.assertRaises(ValueError):
+            # Fails because we prepared a sample with no psi, then asked for psi and did not allow fallback
+            ods.plot_equilibrium_CX(contour_quantity='psi', allow_fallback=False)
+
+        ods = ODS().sample_equilibrium(include_phi=False, include_psi=True)
+        with self.assertRaises(ValueError):
+            # Fails because we prepared a sample with no phi, then asked for phi and did not allow fallback
+            ods.plot_equilibrium_CX(contour_quantity='phi', allow_fallback=False)
+
+        ods = ODS().sample_equilibrium(include_phi=True, include_psi=False, include_q=True)
+        with self.assertRaises(ValueError):
+            # Fails because we prepped sample w/ no psi, then asked for q (uses psi for interp) & didn't allow fallback
+            ods.plot_equilibrium_CX(contour_quantity='q', allow_fallback=False)
+
+        ods = ODS().sample_equilibrium(include_phi=True, include_psi=True, include_q=False)
+        with self.assertRaises(ValueError):
+            # Fails because we prepped sample w/ no q, then asked for q & didn't allow fallback
+            ods.plot_equilibrium_CX(contour_quantity='q', allow_fallback=False)
+
+        ods = ODS().sample_equilibrium(include_phi=False, include_psi=False, include_q=False)
+        with self.assertRaises(ValueError):
+            # Fails because we prepared a sample with no 2D equilibrium data at all and did not allow fallback
+            # Fallback in this case would allow an abort without raising an error
+            ods.plot_equilibrium_CX(allow_fallback=False)
+
+        ods = ODS().sample_equilibrium(include_phi=True, include_psi=True, include_q=True)
+        with self.assertRaises(ValueError):
+            # Fails because we ask for junk. Allow fallback so the ValueError isn't raised due to missing data.
+            ods.plot_equilibrium_CX(contour_quantity='blahblahblah hrrrnggg! EEEEK!!', allow_fallback=True)
+
+    def test_eqcx_slices(self):
+        """Test dealing with different time indices, including getting wall from a different slice than the eq"""
+        ods2 = ODS().sample_equilibrium(time_index=0, include_wall=True)
+        ods2.sample_equilibrium(time_index=1, include_wall=False).plot_equilibrium_CX()  # Get wall from slice 0
+        # Test for missing wall
+        pyplot.figure('TestOmasPlot.test_eqcx missing wall')
+        ODS().sample_equilibrium(include_profiles=True, include_phi=False, include_wall=False).plot_equilibrium_CX()
+
+    def test_eqcx_resample(self):
+        """Test the sf (scaling factor) option"""
+        self.ods.plot_equilibrium_CX(sf=1)
+        self.ods.plot_equilibrium_CX(sf=3)
 
     def test_eq_summary(self):
         ods2 = ODS().sample_equilibrium(include_phi=False)
         ods3 = ODS().sample_equilibrium(include_profiles=True, include_phi=False, include_wall=True)
-        ods2.plot_equilibrium_summary(fig=plt.gcf(), label='label test')
-        ods3.plot_equilibrium_summary(fig=plt.figure('TestOmasPlot.test_eq_summary with rho'))
+        ods2.plot_equilibrium_summary(fig=pyplot.gcf(), label='label test')
+        ods3.plot_equilibrium_summary(fig=pyplot.figure('TestOmasPlot.test_eq_summary with rho'))
 
     def test_core_profiles(self):
         ods2 = copy.deepcopy(self.ods)
         ods2.sample_core_profiles()
-        ods2.plot_core_profiles_summary(fig=plt.gcf())
+        ods2.plot_core_profiles_summary(fig=pyplot.gcf())
         ods2.plot_core_profiles_summary(
-            fig=plt.figure('TestOmasPlot.test_core_profiles totals only'), show_thermal_fast_breakdown=False,
+            fig=pyplot.figure('TestOmasPlot.test_core_profiles totals only'), show_thermal_fast_breakdown=False,
             show_total_density=True)
         ods2.plot_core_profiles_summary(
-            fig=plt.figure('TestOmasPlot.test_core_profiles total and breakdown'), show_thermal_fast_breakdown=True,
+            fig=pyplot.figure('TestOmasPlot.test_core_profiles total and breakdown'), show_thermal_fast_breakdown=True,
             show_total_density=True)
         ods2.plot_core_profiles_summary(
-            fig=plt.figure('TestOmasPlot.test_core_profiles no combine temp/dens'), combine_dens_temps=False)
+            fig=pyplot.figure('TestOmasPlot.test_core_profiles no combine temp/dens'), combine_dens_temps=False)
 
     def test_core_pressure(self):
         ods2 = copy.deepcopy(self.ods)
@@ -251,6 +298,9 @@ class TestOmasPlot(unittest.TestCase):
         ts_ods = ts_ods.sample_thomson_scattering()
         for i, lab in enumerate([2, 3, 5, 7]):
             ts_ods.plot_overlay(thomson_scattering=dict(labelevery=lab, notesize=10 + i * 2 + lab, color='k'))
+        ts_ods.plot_overlay(
+            thomson_scattering=dict(labelevery=2, notesize=9, color='b', label_ha='right', label_va='top')
+        )
 
     # Charge exchange overlay
     def test_cer_overlay(self):
@@ -333,6 +383,84 @@ class TestOmasPlot(unittest.TestCase):
         # Test without equilibrium data: can't use magnetic axis to help decide how to align labels
         ODS().sample_gas_injection().plot_overlay(thomson_scattering=False, gas_injection=True)
 
+    # Langmuir probes overlays
+    def test_langmuir_probes_embedded_overlay(self):
+        """Tests method for plotting overlay of embedded LPs"""
+        # Add sample data
+        lp_ods = ODS()
+        lp_ods.sample_wall()  # The wall is used to decide label alignment
+        lp_ods.sample_langmuir_probes()
+
+        # Basic overlay
+        lp_ods.plot_overlay(thomson_scattering=False, langmuir_probes=True)
+
+        # Overlay with customizations
+        lp_ods.plot_overlay(
+            thomson_scattering=False,
+            langmuir_probes=dict(colors='r', label_ha='left', label_va='top', embedded_probes=['donkey!', 'zzz']),
+        )
+
+        # Direct call
+        lp_ods.plot_langmuir_probes_overlay()
+        # Empty ODS / graceful failure
+        ODS().plot_langmuir_probes_overlay()
+        # No wall data for helping align labels
+        ODS().sample_langmuir_probes().plot_overlay(thomson_scattering=False, langmuir_probes=True)
+
+    def test_position_control_overlay(self):
+        """Tests method for plotting overlay of position_control data"""
+
+        pc_ods = ODS()
+
+        # No data; have to abort
+        pc_ods.plot_overlay(thomson_scattering=False, position_control=True)
+
+        # Add sample data
+        pc_ods.sample_pulse_schedule()
+
+        # Basic test
+        pc_ods.plot_overlay(thomson_scattering=False, position_control=True)
+
+        # Multi-time
+        pc_ods.plot_overlay(thomson_scattering=False, position_control=dict(t=[0.5, 2.3]))
+
+        # Now with equilibrium data (magnetic axis position affects label alignment
+        pc_ods.sample_equilibrium()
+        pc_ods.plot_overlay(thomson_scattering=False, position_control=dict(t=2.3, show_measured_xpoint=True))
+        pc_ods.sample_equilibrium(include_xpoint=True)
+        pc_ods.plot_overlay(thomson_scattering=False, position_control=dict(t=2.3, show_measured_xpoint=True))
+        pc_ods.sample_equilibrium(include_xpoint=True, time_index=1)
+        pc_ods.plot_overlay(thomson_scattering=False, position_control=dict(t=2.3, show_measured_xpoint=True))
+
+        # Call the method itself
+        pc_ods.plot_position_control_overlay()
+
+        # Call with mask
+        nb = len(pc_ods['pulse_schedule']['position_control']['boundary_outline'])
+        nx = len(pc_ods['pulse_schedule']['position_control']['x_point'])
+        ns = len(pc_ods['pulse_schedule']['position_control']['strike_point'])
+        nn = nb + nx + ns
+        mask = [False] * nn
+        mask[nn // 2] = True
+        mask[0] = True
+        mask[-1] = True
+        pc_ods.plot_position_control_overlay(mask=mask)
+        # Force the overlay function to extend the mask by giving it one that's too small
+        small_mask = mask[:-2]
+        small_mask[-1] = True
+        pc_ods.plot_position_control_overlay(mask=small_mask)
+
+    def test_pulse_schedule_overlay(self):
+        """
+        Tests method for plotting overlay of pulse_schedule data.
+        The main item here is position_control, which has its own test. So, this will be short.
+        """
+        import time
+        pc_ods = ODS()
+        pc_ods.sample_pulse_schedule()
+        pc_ods.plot_overlay(thomson_scattering=False, pulse_schedule=dict(timing_ref=time.time()))
+        pc_ods.plot_pulse_schedule_overlay()
+
 
 if __name__ == '__main__':
     suite = unittest.TestLoader().loadTestsFromTestCase(TestOmasPlot)
