@@ -11,18 +11,13 @@ Test script for omas/omas_utils.py
 -------
 """
 
-# Basic imports
 from __future__ import print_function, division, unicode_literals
-import os
 import unittest
+import os
 import numpy
 import warnings
 import copy
 
-# Plot imports
-import matplotlib as mpl
-from matplotlib import pyplot as plt
-
 # OMAS imports
 from omas import *
 from omas.omas_utils import *
@@ -33,70 +28,42 @@ class TestOmasUtils(unittest.TestCase):
     Test suite for omas_utils.py
     """
 
-    # Flags to edit while testing
-    verbose = False  # Spammy, but occasionally useful for debugging a weird problem
-
     # Sample data for use in tests
     specific_test_version = '3.18.0'
 
-    # Utilities for this test
-    def printv(self, *arg):
-        """Utility for tests to use"""
-        if self.verbose:
-            print(*arg)
-
-    def setUp(self):
-        test_id = self.id()
-        test_name = '.'.join(test_id.split('.')[-2:])
-        self.printv('{}...'.format(test_name))
-
-    def tearDown(self):
-        test_name = '.'.join(self.id().split('.')[-2:])
-        self.printv('    {} done.'.format(test_name))
-
     def test_different_ods(self):
         ods = ODS()
         ods2 = ODS()
         ods2.sample_equilibrium()
         diff_eq = different_ods(ods, ods2)
-        self.printv('  diff_eq = {}'.format(diff_eq))
         assert isinstance(diff_eq, list)
         assert ('equilibrium' in ' '.join(diff_eq)) or ('wall' in ' '.join(diff_eq))
         ods3 = copy.deepcopy(ods2)
         assert different_ods(ods2, ods3) is False
         ods3.sample_core_profiles()
-        diff_prof = different_ods(ods2, ods3)
-        self.printv('  diff_prof = {}'.format(diff_prof))
+        diff_prof = ods2.diff(ods3)
         assert isinstance(diff_prof, list)
         assert isinstance(different_ods(ods3, ods2), list)
         assert 'core_profiles' in ' '.join(diff_prof)
         ods2.sample_core_profiles(include_pressure=False)
         diff_prof2 = different_ods(ods3, ods2)
-        self.printv('  diff_prof2 = {}'.format(diff_prof2))
         assert isinstance(diff_prof2, list)
         assert 'core_profiles' in ' '.join(diff_prof2)
         ods2.sample_core_profiles()
         ods2['core_profiles.profiles_1d.0.electrons.density_thermal'][0] = 1.5212
-        diff_prof3 = different_ods(ods2, ods3)
-        self.printv('  diff_prof3 = {}'.format(diff_prof3))
+        diff_prof3 = ods2.diff(ods3)
         assert isinstance(diff_prof3, list)
         assert 'value' in ' '.join(diff_prof3)
         ods2.sample_core_profiles()
-        ods2['core_profiles.profiles_1d.0.ion.0.element.0.a'] = 9.
+        ods2['core_profiles.profiles_1d.0.ion.0.element.0.a'] = 2
         diff_prof4 = different_ods(ods2, ods3)
-        assert isinstance(diff_prof4, list)
-        assert 'type' in ' '.join(diff_prof4)
+        assert not diff_prof4
         ods2.sample_core_profiles()
         ods2['core_profiles.code.name'] = 'fake name 1'
         ods3['core_profiles.code.name'] = 'fake name 2'
         diff_prof5 = different_ods(ods2, ods3)
-        self.printv('  diff_prof5 = {}'.format(diff_prof5))
         assert isinstance(diff_prof5, list)
         assert 'name' in ' '.join(diff_prof5)
-        ods3['core_profiles.code.name'] = numpy.array([2, 3, 4])
-        assert isinstance(different_ods(ods2, ods3), list)
-        ods2['core_profiles.code.name'] = uarray(numpy.array([2, 3, 4]), numpy.array([1, 1, 1]))
-        assert isinstance(different_ods(ods2, ods3), list)
 
     def test_printe(self):
         printe('printe_test,', end='')
@@ -124,7 +91,6 @@ class TestOmasUtils(unittest.TestCase):
         # Exception handling and coping with problems
         self.assertRaises(TypeError, closest_index, 5, 5)  # First arg is not a list --> TypeError
         self.assertRaises(TypeError, closest_index, [1, 2, 3], 'string_not_number')
-        assert closest_index([1, 2, 3], [3]) == 2  # Should use first element of second arg if it's not a scalar
         self.assertRaises(TypeError, closest_index, [1, 2, 3], [3, 2, 1])  # Can't call w/ list as 2nd arg unless len=1
 
     def test_list_structures(self):  # Also tests dict_structures
